@@ -1,6 +1,8 @@
 from pathlib import Path
 import sys
+import platform
 import yaml
+import json
 from yaml.parser import ParserError
 import argparse
 import logging
@@ -9,10 +11,15 @@ from typing import Callable, Iterable, List
 from spectacles import __version__
 from spectacles.runner import Runner
 from spectacles.client import LookerClient
-from spectacles.exceptions import SpectaclesException, GenericValidationError
+from spectacles.exceptions import (
+    LookerApiError,
+    SpectaclesException,
+    GenericValidationError,
+)
 from spectacles.logger import GLOBAL_LOGGER as logger, set_file_handler
 import spectacles.printer as printer
 import spectacles.tracking as tracking
+from spectacles.utils import log_duration
 
 
 class ConfigFileAction(argparse.Action):
@@ -71,7 +78,7 @@ class YamlConfigAction(ConfigFileAction):
             with Path(path).open("r") as file:
                 return yaml.safe_load(file)
         except (FileNotFoundError, ParserError) as error:
-            raise argparse.ArgumentError(self, error)
+            raise argparse.ArgumentError(self, str(error))
 
 
 class EnvVarAction(argparse.Action):
@@ -145,6 +152,23 @@ def handle_exceptions(function: Callable) -> Callable:
             return function(*args, **kwargs)
         except GenericValidationError as error:
             sys.exit(error.exit_code)
+        except LookerApiError as error:
+            logger.error(
+                f"\n{error}\n\n"
+                + printer.dim(
+                    "Run in verbose mode (-v) or check your log file to see the full "
+                    "response from the Looker API. "
+                    "For support, please create an issue at "
+                    "https://github.com/spectacles-ci/spectacles/issues"
+                )
+                + "\n"
+            )
+            looker_api_response = json.dumps(error.looker_api_response, indent=2)
+            logger.debug(
+                f"Spectacles received a {error.status} response code from "
+                f"the Looker API with the following details: {looker_api_response}\n"
+            )
+            sys.exit(error.exit_code)
         except SpectaclesException as error:
             logger.error(
                 f"\n{error}\n\n"
@@ -178,8 +202,21 @@ def handle_exceptions(function: Callable) -> Callable:
 @handle_exceptions
 def main():
     """Runs main function. This is the entry point."""
+    if sys.version_info < (3, 7):
+        raise SpectaclesException(
+            name="insufficient-python-version",
+            title="Spectacles requires Python 3.7 or higher.",
+            detail="The current Python version is %s." % platform.python_version(),
+        )
     parser = create_parser()
     args = parser.parse_args()
+
+    # Normally would be cleaner to handle this with an argparse mutually exclusive
+    # group, but this doesn't work with --commit-ref and --remote-reset also needing
+    # to be mutually exclusive, so raise the error manually.
+    if getattr(args, "branch", None) and getattr(args, "commit_ref", None):
+        parser.error("argument --commit-ref not allowed with argument --branch")
+
     for handler in logger.handlers:
         handler.setLevel(args.log_level)
 
@@ -214,23 +251,41 @@ def main():
             args.api_version,
             args.mode,
             args.remote_reset,
-            args.import_projects,
             args.concurrency,
             args.commit_ref,
+            args.profile,
+            args.runtime_threshold,
         )
     elif args.command == "assert":
         run_assert(
             args.project,
             args.branch,
+            args.explores,
+            args.exclude,
             args.base_url,
             args.client_id,
             args.client_secret,
             args.port,
             args.api_version,
             args.remote_reset,
-            args.import_projects,
             args.commit_ref,
         )
+    elif args.command == "content":
+        run_content(
+            args.project,
+            args.branch,
+            args.explores,
+            args.exclude,
+            args.base_url,
+            args.client_id,
+            args.client_secret,
+            args.port,
+            args.api_version,
+            args.remote_reset,
+            args.commit_ref,
+            args.incremental,
+            args.exclude_personal,
+        )
 
     if not args.do_not_track:
         tracking.track_invocation_end(
@@ -257,6 +312,7 @@ def create_parser() -> argparse.ArgumentParser:
     _build_connect_subparser(subparser_action, base_subparser)
     _build_sql_subparser(subparser_action, base_subparser)
     _build_assert_subparser(subparser_action, base_subparser)
+    _build_content_subparser(subparser_action, base_subparser)
     return parser
 
 
@@ -285,14 +341,14 @@ def _build_base_subparser() -> argparse.ArgumentParser:
         action=EnvVarAction,
         env_var="LOOKER_CLIENT_ID",
         required=True,
-        help="The client ID of the Looker user that spectacles will authenticate as.",
+        help="The client ID of the Looker user that Spectacles will authenticate as.",
     )
     base_subparser.add_argument(
         "--client-secret",
         action=EnvVarAction,
         env_var="LOOKER_CLIENT_SECRET",
         required=True,
-        help="The client secret of the Looker user that spectacles \
+        help="The client secret of the Looker user that Spectacles \
             will authenticate as.",
     )
     base_subparser.add_argument(
@@ -300,8 +356,7 @@ def _build_base_subparser() -> argparse.ArgumentParser:
         type=int,
         action=EnvVarAction,
         env_var="LOOKER_PORT",
-        default=19999,
-        help="The port of your Looker instance’s API. The default is port 19999.",
+        help="The port of your Looker instance’s API. The default is port 443 (HTTPS) for GCP-hosted instances and 19999 for legacy instances.",
     )
     base_subparser.add_argument(
         "--api-version",
@@ -318,7 +373,7 @@ def _build_base_subparser() -> argparse.ArgumentParser:
         dest="log_level",
         const=logging.DEBUG,
         default=logging.INFO,
-        help="Display debug logging during spectacles execution. \
+        help="Display debug logging during Spectacles execution. \
             Useful for debugging and making bug reports.",
     )
     base_subparser.add_argument(
@@ -381,16 +436,25 @@ def _build_validator_subparser(
         "--branch",
         action=EnvVarAction,
         env_var="LOOKER_GIT_BRANCH",
-        required=True,
         help="The branch of your project that Spectacles will use to run queries.",
     )
     base_subparser.add_argument(
-        "--import-projects",
-        action=EnvVarStoreTrueAction,
-        env_var="SPECTACLES_IMPORT_PROJECTS",
-        help="When set to true, the SQL Validator will create temporary branches \
-            that are clones of master for any project that is a local dependency of the \
-            of the project being tested. These branches are deleted at the end of the run.",
+        "--explores",
+        nargs="+",
+        default=["*/*"],
+        help="Specify the explores Spectacles should test. \
+            List of strings in 'model_name/explore_name' format. \
+            The '*' wildcard selects all models or explores. For instance,\
+            'model_name/*' would select all explores in the 'model_name' model.",
+    )
+    base_subparser.add_argument(
+        "--exclude",
+        nargs="+",
+        default=[],
+        help="Specify the explores Spectacles should exclude when testing. \
+            List of strings in 'model_name/explore_name' format. \
+            The '*' wildcard excludes all models or explores. For instance,\
+            'model_name/*' would select all explores in the 'model_name' model.",
     )
     group = base_subparser.add_mutually_exclusive_group()
     group.add_argument(
@@ -435,24 +499,6 @@ def _build_sql_subparser(
 
     _build_validator_subparser(subparser_action, subparser)
 
-    subparser.add_argument(
-        "--explores",
-        nargs="+",
-        default=["*/*"],
-        help="Specify the explores spectacles should test. \
-            List of selector strings in 'model_name/explore_name' format. \
-            The '*' wildcard selects all models or explores. For instance,\
-            'model_name/*' would select all explores in the 'model_name' model.",
-    )
-    subparser.add_argument(
-        "--exclude",
-        nargs="+",
-        default=[],
-        help="Specify the explores spectacles should exclude when testing. \
-            List of selector strings in 'model_name/explore_name' format. \
-            The '*' wildcard excludes all models or explores. For instance,\
-            'model_name/*' would select all explores in the 'model_name' model.",
-    )
     subparser.add_argument(
         "--mode",
         choices=["batch", "single", "hybrid"],
@@ -460,7 +506,7 @@ def _build_sql_subparser(
         help="Specify the mode the SQL validator should run.\
             In single-dimension mode, the SQL validator will run one query \
             per dimension. In batch mode, the SQL validator will create one \
-            query per explore. In hybrid mode, the SQL validator will run in \
+            query per explore. In hybrid mode, the SQL validator will first run in \
             batch mode and then run errored explores in single-dimension mode.",
     )
     subparser.add_argument(
@@ -470,6 +516,24 @@ def _build_sql_subparser(
         help="Specify how many concurrent queries you want to have running \
             against your data warehouse. The default is 10.",
     )
+    subparser.add_argument(
+        "-p",
+        "--profile",
+        action="store_true",
+        help=(
+            "After validation, display queries that took longer than the runtime "
+            "threshold (5 seconds by default) to complete."
+        ),
+    )
+    subparser.add_argument(
+        "--runtime-threshold",
+        type=int,
+        default=5,
+        help=(
+            "When profiling, only display queries that ran longer than this value in "
+            "seconds."
+        ),
+    )
 
 
 def _build_assert_subparser(
@@ -493,6 +557,29 @@ def _build_assert_subparser(
     _build_validator_subparser(subparser_action, subparser)
 
 
+def _build_content_subparser(
+    subparser_action: argparse._SubParsersAction,
+    base_subparser: argparse.ArgumentParser,
+) -> None:
+    subparser = subparser_action.add_parser(
+        "content", parents=[base_subparser], help="Run Looker content validation."
+    )
+
+    subparser.add_argument(
+        "--incremental",
+        action="store_true",
+        help="Only display errors which are not present on the production branch.",
+    )
+
+    subparser.add_argument(
+        "--exclude-personal",
+        action="store_true",
+        help="Exclude errors found in content in personal folders.",
+    )
+
+    _build_validator_subparser(subparser_action, subparser)
+
+
 def run_connect(
     base_url: str, client_id: str, client_secret: str, port: int, api_version: float
 ) -> None:
@@ -500,31 +587,76 @@ def run_connect(
     LookerClient(base_url, client_id, client_secret, port, api_version)
 
 
-def run_assert(
+@log_duration
+def run_content(
     project,
     branch,
+    explores,
+    excludes,
+    base_url,
+    client_id,
+    client_secret,
+    port,
+    api_version,
+    remote_reset,
+    commit_ref,
+    incremental,
+    exclude_personal,
+) -> None:
+    runner = Runner(
+        base_url, project, client_id, client_secret, port, api_version, remote_reset
+    )
+    results = runner.validate_content(
+        branch, commit_ref, explores, excludes, incremental, exclude_personal
+    )
+
+    for test in sorted(results["tested"], key=lambda x: (x["model"], x["explore"])):
+        message = f"{test['model']}.{test['explore']}"
+        printer.print_validation_result(passed=test["passed"], source=message)
+
+    errors = sorted(
+        results["errors"],
+        key=lambda x: (x["model"], x["explore"], x["metadata"]["field_name"]),
+    )
+    if errors:
+        for error in errors:
+            printer.print_content_error(
+                error["model"],
+                error["explore"],
+                error["message"],
+                error["metadata"]["content_type"],
+                error["metadata"]["space"],
+                error["metadata"]["title"],
+                error["metadata"]["url"],
+            )
+        logger.info("")
+        raise GenericValidationError
+    else:
+        logger.info("")
+
+
+@log_duration
+def run_assert(
+    project,
+    branch,
+    explores,
+    exclude,
     base_url,
     client_id,
     client_secret,
     port,
     api_version,
     remote_reset,
-    import_projects,
     commit_ref,
 ) -> None:
     runner = Runner(
-        base_url,
-        project,
-        branch,
-        client_id,
-        client_secret,
-        port,
-        api_version,
-        remote_reset,
-        import_projects,
-        commit_ref,
+        base_url, project, client_id, client_secret, port, api_version, remote_reset
     )
-    results = runner.validate_data_tests()
+    results = runner.validate_data_tests(branch, commit_ref, explores, exclude)
+
+    for test in sorted(results["tested"], key=lambda x: (x["model"], x["explore"])):
+        message = f"{test['model']}.{test['explore']}"
+        printer.print_validation_result(passed=test["passed"], source=message)
 
     errors = sorted(
         results["errors"],
@@ -537,6 +669,7 @@ def run_assert(
                 error["explore"],
                 error["metadata"]["test_name"],
                 error["message"],
+                error["metadata"]["lookml_url"],
             )
         logger.info("")
         raise GenericValidationError
@@ -544,6 +677,7 @@ def run_assert(
         logger.info("")
 
 
+@log_duration
 def run_sql(
     log_dir,
     project,
@@ -557,22 +691,14 @@ def run_sql(
     api_version,
     mode,
     remote_reset,
-    import_projects,
     concurrency,
     commit_ref,
+    profile,
+    runtime_threshold,
 ) -> None:
     """Runs and validates the SQL for each selected LookML dimension."""
     runner = Runner(
-        base_url,
-        project,
-        branch,
-        client_id,
-        client_secret,
-        port,
-        api_version,
-        remote_reset,
-        import_projects,
-        commit_ref,
+        base_url, project, client_id, client_secret, port, api_version, remote_reset
     )
 
     def iter_errors(lookml: List) -> Iterable:
@@ -580,7 +706,21 @@ def run_sql(
             if item.errored:
                 yield item
 
-    results = runner.validate_sql(explores, exclude, mode, concurrency)
+    results = runner.validate_sql(
+        branch,
+        commit_ref,
+        explores,
+        exclude,
+        mode,
+        concurrency,
+        profile,
+        runtime_threshold,
+    )
+
+    for test in sorted(results["tested"], key=lambda x: (x["model"], x["explore"])):
+        message = f"{test['model']}.{test['explore']}"
+        printer.print_validation_result(passed=test["passed"], source=message)
+
     errors = sorted(
         results["errors"],
         key=lambda x: (x["model"], x["explore"], x["metadata"].get("dimension")),
@@ -592,16 +732,20 @@ def run_sql(
                 model=error["model"],
                 explore=error["explore"],
                 message=error["message"],
-                sql=error["test"],
+                sql=error["metadata"]["sql"],
                 log_dir=log_dir,
                 dimension=error["metadata"].get("dimension"),
                 lookml_url=error["metadata"].get("lookml_url"),
             )
         if mode == "batch":
             logger.info(
-                f"\n\nTo determine the exact dimensions responsible for {'this error' if len(errors) == 1 else 'these errors'}, "
-                f"you can re-run \nSpectacles in single-dimension mode, with `--mode single`.\n\n"
-                "You can also run this original validation with `--mode hybrid` to do this automatically."
+                printer.dim(
+                    "\n\nTo determine the exact dimensions responsible for "
+                    f"{'this error' if len(errors) == 1 else 'these errors'}, "
+                    "you can re-run \nSpectacles in single-dimension mode, "
+                    "with `--mode single`.\n\nYou can also run this original "
+                    "validation with `--mode hybrid` to do this automatically."
+                )
             )
 
         logger.info("")
