@@ -48,7 +48,7 @@ class LookerClient:
         base_url: str,
         client_id: str,
         client_secret: str,
-        port: int = 19999,
+        port: Optional[int] = None,
         api_version: float = 3.1,
     ):
         supported_api_versions = [3.1]
@@ -64,7 +64,17 @@ class LookerClient:
             )
 
         self.base_url: str = base_url.rstrip("/")
-        self.api_url: str = f"{self.base_url}:{port}/api/{api_version}/"
+        if port is None and self.base_url.endswith("cloud.looker.com"):
+            # GCP-hosted instance, so use default port of 443 with HTTPS
+            if not self.base_url.startswith("https"):
+                raise SpectaclesException(
+                    name="invalid-base-url",
+                    title="Looker instance base URL is not valid.",
+                    detail="The URL must be an HTTPS URL.",
+                )
+            self.api_url: str = f"{self.base_url}/api/{api_version}/"
+        else:
+            self.api_url = f"{self.base_url}:{port or 19999}/api/{api_version}/"
         self.client_id: str = client_id
         self.client_secret: str = client_secret
         self.api_version: float = api_version
@@ -86,6 +96,7 @@ class LookerClient:
 
         url = utils.compose_url(self.api_url, path=["login"])
         body = {"client_id": self.client_id, "client_secret": self.client_secret}
+        self.session.auth = NullAuth()
         # This should not use `self.post` or it will create a recursive loop
         response = self.session.post(url=url, data=body, timeout=TIMEOUT_SEC)
         try:
@@ -166,11 +177,37 @@ class LookerClient:
 
         return response.json()["looker_release_version"]
 
-    def update_workspace(self, project: str, workspace: str) -> None:
-        """Updates the session workspace.
+    def get_workspace(self) -> str:
+        """Gets the session workspace.
 
         Args:
             project: Name of the Looker project to use.
+
+        Returns:
+            str: The session workspace, dev or production.
+        """
+        logger.debug("Getting the workspace in use by this session")
+        url = utils.compose_url(self.api_url, path=["session"])
+        response = self.get(url=url, timeout=TIMEOUT_SEC)
+        try:
+            response.raise_for_status()
+        except requests.exceptions.HTTPError:
+            raise LookerApiError(
+                name="unable-to-get-workspace",
+                title="Couldn't get the workspace.",
+                status=response.status_code,
+                detail=(
+                    "Unable to get the workspace in use by this session. "
+                    "Please try again."
+                ),
+                response=response,
+            )
+        return response.json()["workspace_id"]
+
+    def update_workspace(self, workspace: str) -> None:
+        """Updates the session workspace.
+
+        Args:
             workspace: The workspace to switch to, either 'production' or 'dev'
         """
         logger.debug(f"Updating session to use the {workspace} workspace")
@@ -227,7 +264,7 @@ class LookerClient:
             project: Name of the Looker project to use.
             branch: Name of the Git branch to check out.
         """
-        logger.debug(f"Setting Git branch to '{branch}'")
+        logger.debug(f"Setting project '{project}' branch to '{branch}'")
         url = utils.compose_url(self.api_url, path=["projects", project, "git_branch"])
         body = {"name": branch}
         response = self.put(url=url, json=body, timeout=TIMEOUT_SEC)
@@ -266,7 +303,7 @@ class LookerClient:
                 title="Couldn't checkout Git branch.",
                 status=response.status_code,
                 detail=(
-                    "Unable to reset local Git branch"
+                    "Unable to reset local Git branch "
                     "to match remote. Please try again."
                 ),
                 response=response,
@@ -341,7 +378,7 @@ class LookerClient:
         full_response = self.get_active_branch(project)
         return full_response["name"]
 
-    def create_branch(self, project: str, branch: str, ref: str = "origin/master"):
+    def create_branch(self, project: str, branch: str, ref: Optional[str] = None):
         """Creates a branch in the given project.
 
         Args:
@@ -349,11 +386,25 @@ class LookerClient:
             branch: Name of the branch to create.
             ref: The ref to create the branch from.
         """
-        logger.debug(
-            f"Creating branch '{branch}' on project '{project}' with ref '{ref}'"
+        body = {"name": branch}
+        message = f"Creating branch '{branch}' on project '{project}'"
+        detail = (
+            f"Unable to create branch '{branch}' "
+            f"in project '{project}'. "
+            "Confirm the branch doesn't already exist and try again."
         )
 
-        body = {"name": branch, "ref": ref}
+        if ref:
+            body["ref"] = ref
+            message += f" with ref '{ref}'"
+            detail = (
+                f"Unable to create branch '{branch}' "
+                f"in project '{project}' using ref '{ref}'. "
+                "Confirm the branch doesn't already exist and try again."
+            )
+
+        logger.debug(message)
+
         url = utils.compose_url(self.api_url, path=["projects", project, "git_branch"])
         response = self.post(url=url, json=body, timeout=TIMEOUT_SEC)
 
@@ -364,16 +415,15 @@ class LookerClient:
                 name="unable-to-create-branch",
                 title="Couldn't create new Git branch.",
                 status=response.status_code,
-                detail=(
-                    f"Unable to create branch '{branch}' "
-                    f"in project '{project}' using ref '{ref}'. "
-                    "Confirm the branch doesn't already exist and try again."
-                ),
+                detail=detail,
                 response=response,
             )
 
-    def update_branch(self, project: str, branch: str, ref: str = "origin/master"):
-        """Updates a branch to the ref prodvided.
+    def hard_reset_branch(self, project: str, branch: str, ref: str):
+        """Hard resets a branch to the ref prodvided.
+
+        DANGER: hard reset will be force pushed to the remote. Unsaved changes and
+            commits may be permanently lost.
 
         Args:
             project: Name of the Looker project to use.
@@ -381,7 +431,7 @@ class LookerClient:
             ref: The ref to update the branch from.
         """
         logger.debug(
-            f"Updating branch '{branch}' on project '{project}' to ref '{ref}'"
+            f"Hard resetting branch '{branch}' on project '{project}' to ref '{ref}'"
         )
 
         body = {"name": branch, "ref": ref}
@@ -515,7 +565,7 @@ class LookerClient:
 
         return response.json()
 
-    def get_lookml_models(self) -> List[JsonDict]:
+    def get_lookml_models(self, fields: List = []) -> List[JsonDict]:
         """Gets all models and explores from the LookmlModel endpoint.
 
         Returns:
@@ -523,7 +573,12 @@ class LookerClient:
 
         """
         logger.debug(f"Getting all models and explores from {self.base_url}")
-        url = utils.compose_url(self.api_url, path=["lookml_models"])
+
+        params = {}
+        if fields:
+            params["fields"] = fields
+
+        url = utils.compose_url(self.api_url, path=["lookml_models"], params=params)
         response = self.get(url=url, timeout=TIMEOUT_SEC)
         try:
             response.raise_for_status()
@@ -551,8 +606,11 @@ class LookerClient:
 
         """
         logger.debug(f"Getting all dimensions from explore {explore}")
+        params = {"fields": ["fields"]}
         url = utils.compose_url(
-            self.api_url, path=["lookml_models", model, "explores", explore]
+            self.api_url,
+            path=["lookml_models", model, "explores", explore],
+            params=params,
         )
         response = self.get(url=url, timeout=TIMEOUT_SEC)
         try:
@@ -572,7 +630,9 @@ class LookerClient:
         return response.json()["fields"]["dimensions"]
 
     @backoff.on_exception(backoff.expo, (Timeout,), max_tries=2)
-    def create_query(self, model: str, explore: str, dimensions: List[str]) -> Dict:
+    def create_query(
+        self, model: str, explore: str, dimensions: List[str], fields: List = []
+    ) -> Dict:
         """Creates a Looker async query for one or more specified dimensions.
 
         The query created is a SELECT query, selecting all dimensions specified for a
@@ -596,7 +656,12 @@ class LookerClient:
             "limit": 0,
             "filter_expression": "1=2",
         }
-        url = utils.compose_url(self.api_url, path=["queries"])
+
+        params = {}
+        if fields:
+            params["fields"] = fields
+
+        url = utils.compose_url(self.api_url, path=["queries"], params=params)
         response = self.post(url=url, json=body, timeout=TIMEOUT_SEC)
         try:
             response.raise_for_status()
@@ -642,7 +707,8 @@ class LookerClient:
         # Using old-style string formatting so that strings are formatted lazily
         logger.debug("Starting query %d", query_id)
         body = {"query_id": query_id, "result_format": "json_detail"}
-        url = utils.compose_url(self.api_url, path=["query_tasks"])
+        params = {"fields": ["id"]}
+        url = utils.compose_url(self.api_url, path=["query_tasks"], params=params)
 
         response = self.post(
             url=url, json=body, params={"cache": "false"}, timeout=TIMEOUT_SEC
@@ -759,3 +825,12 @@ class LookerClient:
 
         result = response.json()
         return result
+
+
+class NullAuth(requests.auth.AuthBase):
+    """A custom auth class which ensures requests does not override authorization
+    headers with netrc file credentials if present.
+    """
+
+    def __call__(self, r):
+        return r
